# make a textile file into a kanban board
#
# input a file in the form specified by example.textile
# result: an internal representation of the board good for being displayed, which
# includes:
#
# - project name
# - list of workflow states defined in the file
# - list of swimlanes defined in the files
# - entries defined in the file organized in a two-dimensional matrix:
#
#      (status, swimlane)
#
#   (so that they can be placed in the kanban board)
#
#   the entries are stored with a hash of arrays. (status, swimlane) are transformed
#   into a unique key (with string concatenation) and the entry added as elements
#   of an array

require "kanboard/card"

class Kanboard::Board
  attr_accessor :project, :statuses, :swimlanes, :all_cards

  def initialize
    @project = ""
    @statuses = []         # all statuses (h2. strings) defined in the input file
    @swimlanes = []        # all unique swim-lane (+ strings) defined in the input file
    @cards = Hash.new      # cards organised by board position
    @all_cards = Array.new # all cards defined in the input file
  end

  def load filename
    current_status = "" # current status

    File.open(filename).each do |line|
      # set the title to the first h1 found in the file
      match = line.match(/^h1\.[ ]+(.+)$/)
      if match and @project == ""
        @project = match[1]
      end

      # look for statuses and add them to the list of statuses.
      # don't bother about duplications
      match = line.match(/^h2\.[ ]+(.+)$/)
      if match
        @statuses << match[1]
        current_status = match[1]
      end

      # look for cards and manage them
      if line.start_with?('- ') # make sure we ignore YAML front matter ("---")
        card = Kanboard::Card.new(@project, @status, line)
        swimlane = card.swimlane
        owner = card.owner

        @swimlanes << swimlane unless @swimlanes.include? swimlane

        position = hash(current_status, swimlane)
        @cards[position] = Array.new unless @cards[position]
        @cards[position] << card

        @all_cards << card
      end
    end
  end

  def cards(status, swimlane)
    @cards[hash(status, swimlane)] || Array.new  # so that we always return an array
  end

  #
  # return all the owners mentioned in the input file
  #
  def owners
    @all_cards.map { |c| c.owner }.uniq
  end

  #
  # return all the cards owned by owner
  #
  def cards_of(owner)
    @all_cards.select { |c| c.owner == owner }
  end

  private
  
  #
  # generate an hash which uniquely identifies a cell in the kanban board
  #
  def hash(status, swimlane)
    status + "*" + swimlane
  end

end